@@ -8,10 +8,10 @@ See `the FlickrAPI homepage`_ for more info.
 .. _`the FlickrAPI homepage`: http://flickrapi.sf.net/
 '''
 
-__version__ = '1.0'
+__version__ = '1.1'
 __all__ = ('FlickrAPI', 'IllegalArgumentException', 'FlickrError',
         'XMLNode', 'set_log_level', '__version__')
-__author__ = u'Sybren Stüvel'
+__author__ = u'Sybren St\u00fcvel'.encode('utf-8')
 
 # Copyright (c) 2007 by the respective coders, see
 # http://flickrapi.sf.net/
@@ -55,6 +55,7 @@ from flickrapi.tokencache import TokenCache, SimpleTokenCache
 from flickrapi.xmlnode import XMLNode
 from flickrapi.multipart import Part, Multipart, FilePart
 from flickrapi.exceptions import IllegalArgumentException, FlickrError
+from flickrapi.cache import SimpleCache
 from flickrapi import reportinghttp
 
 logging.basicConfig()
@@ -79,6 +80,37 @@ def make_utf8(dictionary):
     
     return result
         
+def debug(method):
+    '''Method decorator for debugging method calls.
+
+    Using this automatically sets the log level to DEBUG.
+    '''
+
+    LOG.setLevel(logging.DEBUG)
+
+    def debugged(*args, **kwargs):
+        LOG.debug("Call: %s(%s, %s)" % (method.__name__, args,
+            kwargs))
+        result = method(*args, **kwargs)
+        LOG.debug("\tResult: %s" % result)
+        return result
+
+    return debugged
+
+# REST parsers, {format: parser_method, ...}. Fill by using the
+# @rest_parser(format) function decorator
+rest_parsers = {}
+
+def rest_parser(format):
+    '''Function decorator, use this to mark a function as the parser for REST as
+    returned by Flickr.
+    '''
+
+    def decorate_parser(method):
+        rest_parsers[format] = method
+        return method
+
+    return decorate_parser
 
 class FlickrAPI:
     """Encapsulates Flickr functionality.
@@ -96,20 +128,21 @@ class FlickrAPI:
     flickr_upload_form = "/services/upload/"
     flickr_replace_form = "/services/replace/"
 
-    def __init__(self, api_key, secret=None, fail_on_error=True,
-                 username=None, token=None):
+    def __init__(self, api_key, secret=None, fail_on_error=None, username=None,
+            token=None, format='xmlnode', store_token=True, cache=False):
         """Construct a new FlickrAPI instance for a given API key
         and secret.
         
         api_key
-            The API key as obtained from Flickr
+            The API key as obtained from Flickr.
         
         secret
-            The secret belonging to the API key
+            The secret belonging to the API key.
         
         fail_on_error
             If False, errors won't be checked by the FlickrAPI module.
-            True by default, of course.
+            Deprecated, don't use this parameter, just handle the FlickrError
+            exceptions.
         
         username
             Used to identify the appropriate authentication token for a
@@ -117,13 +150,38 @@ class FlickrAPI:
         
         token
             If you already have an authentication token, you can give
-            it here. It won't be stored on disk by the FlickrAPI instance
-        
+            it here. It won't be stored on disk by the FlickrAPI instance.
+
+        format
+            The response format. Use either "xmlnode" or "etree" to get a parsed
+            response, or use any response format supported by Flickr to get an
+            unparsed response from method calls. It's also possible to pass the
+            ``format`` parameter on individual calls.
+
+        store_token
+            Disables the on-disk token cache if set to False (default is True).
+            Use this to ensure that tokens aren't read nor written to disk, for
+            example in web applications that store tokens in cookies.
+
+        cache
+            Enables in-memory caching of FlickrAPI calls - set to ``True`` to
+            use. If you don't want to use the default settings, you can
+            instantiate a cache yourself too:
+
+            >>> f = FlickrAPI(api_key='123')
+            >>> f.cache = SimpleCache(timeout=5, max_entries=100)
         """
         
+        if fail_on_error is not None:
+            LOG.warn("fail_on_error has been deprecated. Remove this "
+                     "parameter and just handle the FlickrError exceptions.")
+        else:
+            fail_on_error = True
+
         self.api_key = api_key
         self.secret = secret
         self.fail_on_error = fail_on_error
+        self.default_format = format
         
         self.__handler_cache = {}
 
@@ -131,17 +189,70 @@ class FlickrAPI:
             # Use a memory-only token cache
             self.token_cache = SimpleTokenCache()
             self.token_cache.token = token
+        elif not store_token:
+            # Use an empty memory-only token cache
+            self.token_cache = SimpleTokenCache()
         else:
             # Use a real token cache
             self.token_cache = TokenCache(api_key, username)
 
+        if cache:
+            self.cache = SimpleCache()
+        else:
+            self.cache = None
+
     def __repr__(self):
         '''Returns a string representation of this object.'''
 
 
         return '[FlickrAPI for key "%s"]' % self.api_key
     __str__ = __repr__
-    
+
+    def trait_names(self):
+        '''Returns a list of method names as supported by the Flickr
+        API. Used for tab completion in IPython.
+        '''
+
+        rsp = self.reflection_getMethods(format='etree')
+
+        def tr(name):
+            '''Translates Flickr names to something that can be called
+            here.
+
+            >>> tr(u'flickr.photos.getInfo')
+            u'photos_getInfo'
+            '''
+            
+            return name[7:].replace('.', '_')
+
+        return [tr(m.text) for m in rsp.getiterator('method')]
+
+    @rest_parser('xmlnode')
+    def parse_xmlnode(self, rest_xml):
+        '''Parses a REST XML response from Flickr into an XMLNode object.'''
+
+        rsp = XMLNode.parse(rest_xml, store_xml=True)
+        if rsp['stat'] == 'ok' or not self.fail_on_error:
+            return rsp
+        
+        err = rsp.err[0]
+        raise FlickrError(u'Error: %(code)s: %(msg)s' % err)
+
+    @rest_parser('etree')
+    def parse_etree(self, rest_xml):
+        '''Parses a REST XML response from Flickr into an ElementTree object.'''
+
+        # Only import it here, to maintain Python 2.4 compatibility
+        import xml.etree.ElementTree
+
+        rsp = xml.etree.ElementTree.fromstring(rest_xml)
+        if rsp.attrib['stat'] == 'ok' or not self.fail_on_error:
+            return rsp
+        
+        err = rsp.find('err')
+        raise FlickrError(u'Error: %s: %s' % (
+            err.attrib['code'], err.attrib['msg']))
+
     def sign(self, dictionary):
         """Calculate the flickr signature for a set of params.
         
@@ -183,11 +294,13 @@ class FlickrAPI:
 
             flickr.auth_getFrob(api_key="AAAAAA")
             xmlnode = flickr.photos_getInfo(photo_id='1234')
+            xmlnode = flickr.photos_getInfo(photo_id='1234', format='xmlnode')
             json = flickr.photos_getInfo(photo_id='1234', format='json')
+            etree = flickr.photos_getInfo(photo_id='1234', format='etree')
         """
 
         # Refuse to act as a proxy for unimplemented special methods
-        if attrib.startswith('__'):
+        if attrib.startswith('_'):
             raise AttributeError("No such attribute '%s'" % attrib)
 
         # Construct the method name and see if it's cached
@@ -195,13 +308,9 @@ class FlickrAPI:
         if method in self.__handler_cache:
             return self.__handler_cache[method]
         
-        url = "http://" + FlickrAPI.flickr_host + FlickrAPI.flickr_rest_form
-
         def handler(**args):
             '''Dynamically created handler for a Flickr API call'''
 
-            explicit_format = 'format' in args
-
             if self.token_cache.token and not self.secret:
                 raise ValueError("Auth tokens cannot be used without "
                                  "API secret")
@@ -210,38 +319,99 @@ class FlickrAPI:
             defaults = {'method': method,
                         'auth_token': self.token_cache.token,
                         'api_key': self.api_key,
-                        'format': 'rest'}
+                        'format': self.default_format}
 
-            for key, default_value in defaults.iteritems():
-                if key not in args:
-                    args[key] = default_value
-                # You are able to remove a default by assigning None
-                if key in args and args[key] is None:
-                    del args[key]
+            args = self.__supply_defaults(args, defaults)
 
-            LOG.debug("Calling %s(%s)" % (method, args))
-
-            post_data = self.encode_and_sign(args)
-
-            flicksocket = urllib.urlopen(url, post_data)
-            data = flicksocket.read()
-            flicksocket.close()
-
-            # Return the raw response when the user requested
-            # a specific format.
-            if explicit_format:
-                return data
-            
-            result = XMLNode.parse(data, True)
-            if self.fail_on_error:
-                FlickrAPI.test_failure(result, True)
-
-            return result
+            return self.__wrap_in_parser(self.__flickr_call,
+                    parse_format=args['format'], **args)
 
         handler.method = method
         self.__handler_cache[method] = handler
         return handler
     
+    def __supply_defaults(self, args, defaults):
+        '''Returns a new dictionary containing ``args``, augmented with defaults
+        from ``defaults``.
+
+        Defaults can be overridden, or completely removed by setting the
+        appropriate value in ``args`` to ``None``.
+
+        >>> f = FlickrAPI('123')
+        >>> f._FlickrAPI__supply_defaults(
+        ...  {'foo': 'bar', 'baz': None, 'token': None},
+        ...  {'baz': 'foobar', 'room': 'door'})
+        {'foo': 'bar', 'room': 'door'}
+        '''
+
+        result = args.copy()
+        for key, default_value in defaults.iteritems():
+            # Set the default if the parameter wasn't passed
+            if key not in args:
+                result[key] = default_value
+
+        for key, value in result.copy().iteritems():
+            # You are able to remove a default by assigning None, and we can't
+            # pass None to Flickr anyway.
+            if result[key] is None:
+                del result[key]
+        
+        return result
+
+    def __flickr_call(self, **kwargs):
+        '''Performs a Flickr API call with the given arguments. The method name
+        itself should be passed as the 'method' parameter.
+        
+        Returns the unparsed data from Flickr::
+
+            data = self.__flickr_call(method='flickr.photos.getInfo',
+                photo_id='123', format='rest')
+        '''
+
+        LOG.debug("Calling %s" % kwargs)
+
+        post_data = self.encode_and_sign(kwargs)
+
+        # Return value from cache if available
+        if self.cache and self.cache.get(post_data):
+            return self.cache.get(post_data)
+
+        url = "http://" + FlickrAPI.flickr_host + FlickrAPI.flickr_rest_form
+        flicksocket = urllib.urlopen(url, post_data)
+        reply = flicksocket.read()
+        flicksocket.close()
+
+        # Store in cache, if we have one
+        if self.cache is not None:
+            self.cache.set(post_data, reply)
+
+        return reply
+    
+    def __wrap_in_parser(self, wrapped_method, parse_format, *args, **kwargs):
+        '''Wraps a method call in a parser.
+
+        The parser will be looked up by the ``parse_format`` specifier. If there
+        is a parser and ``kwargs['format']`` is set, it's set to ``rest``, and
+        the response of the method is parsed before it's returned.
+        '''
+
+        # Find the parser, and set the format to rest if we're supposed to
+        # parse it.
+        if parse_format in rest_parsers and 'format' in kwargs:
+            kwargs['format'] = 'rest'
+
+        LOG.debug('Wrapping call %s(self, %s, %s)' % (wrapped_method, args,
+            kwargs))
+        data = wrapped_method(*args, **kwargs)
+
+        # Just return if we have no parser
+        if parse_format not in rest_parsers:
+            return data
+
+        # Return the parsed data
+        parser = rest_parsers[parse_format]
+        return parser(self, data)
+
     def auth_url(self, perms, frob):
         """Return the authorization URL to get a token.
 
@@ -276,7 +446,6 @@ class FlickrAPI:
 
         return "http://%s%s?%s" % (FlickrAPI.flickr_host, \
             FlickrAPI.flickr_auth_form, encoded)
-        
 
     def upload(self, filename, callback=None, **arg):
         """Upload a file to flickr.
@@ -418,15 +587,15 @@ class FlickrAPI:
             response = urllib2.urlopen(request)
         rspXML = response.read()
 
-        result = XMLNode.parse(rspXML)
-        if self.fail_on_error:
-            FlickrAPI.test_failure(result, True)
-
-        return result
+        return self.parse_xmlnode(rspXML)
 
     @classmethod
     def test_failure(cls, rsp, exception_on_error=True):
         """Exit app if the rsp XMLNode indicates failure."""
+
+        LOG.warn("FlickrAPI.test_failure has been deprecated and will be "
+                 "removed in FlickrAPI version 1.2.")
+
         if rsp['stat'] != "fail":
             return
         
@@ -438,13 +607,23 @@ class FlickrAPI:
 
     @classmethod
     def get_printable_error(cls, rsp):
-        """Return a printed error message string."""
+        """Return a printed error message string of an XMLNode Flickr response."""
+
+        LOG.warn("FlickrAPI.get_printable_error has been deprecated "
+                 "and will be removed in FlickrAPI version 1.2.")
+
         return "%s: error %s: %s" % (rsp.name, \
             cls.get_rsp_error_code(rsp), cls.get_rsp_error_msg(rsp))
 
     @classmethod
     def get_rsp_error_code(cls, rsp):
-        """Return the error code of a response, or 0 if no error."""
+        """Return the error code of an XMLNode Flickr response, or 0 if no
+        error.
+        """
+
+        LOG.warn("FlickrAPI.get_rsp_error_code has been deprecated and will be "
+                 "removed in FlickrAPI version 1.2.")
+
         if rsp['stat'] == "fail":
             return int(rsp.err[0]['code'])
 
@@ -452,7 +631,13 @@ class FlickrAPI:
 
     @classmethod
     def get_rsp_error_msg(cls, rsp):
-        """Return the error message of a response, or "Success" if no error."""
+        """Return the error message of an XMLNode Flickr response, or "Success"
+        if no error.
+        """
+
+        LOG.warn("FlickrAPI.get_rsp_error_msg has been deprecated and will be "
+                 "removed in FlickrAPI version 1.2.")
+
         if rsp['stat'] == "fail":
             return rsp.err[0]['msg']
 
@@ -510,7 +695,7 @@ class FlickrAPI:
         if token:
             LOG.debug("Trying cached token '%s'" % token)
             try:
-                rsp = self.auth_checkToken(auth_token=token)
+                rsp = self.auth_checkToken(auth_token=token, format='xmlnode')
 
                 # see if we have enough permissions
                 tokenPerms = rsp.auth[0].perms[0].text
@@ -525,7 +710,7 @@ class FlickrAPI:
         if not token:
             # get the frob
             LOG.debug("Getting frob for new token")
-            rsp = self.auth_getFrob(auth_token=None)
+            rsp = self.auth_getFrob(auth_token=None, format='xmlnode')
             self.test_failure(rsp)
 
             frob = rsp.frob[0].text
@@ -554,7 +739,7 @@ class FlickrAPI:
         '''
         
         # get a token
-        rsp = self.auth_getToken(frob=frob, auth_token=None)
+        rsp = self.auth_getToken(frob=frob, auth_token=None, format='xmlnode')
         self.test_failure(rsp)
 
         token = rsp.auth[0].token[0].text
@@ -577,3 +762,10 @@ def set_log_level(level):
 
     LOG.setLevel(level)
     flickrapi.tokencache.LOG.setLevel(level)
+
+
+if __name__ == "__main__":
+    print "Running doctests"
+    import doctest
+    doctest.testmod()
+    print "Tests OK"
